import { exec, execSync } from 'child_process';
import { readFileSync } from 'fs';
import path from 'path';

var removeLineBreaks = function (str) {
  return str.replace(/(\r\n|\n|\r)/gm, "")
};

var runGitCommand = function (command, callback) {
  var gitCommand = ['git', command].join(' ');

  if (callback) {
    exec(gitCommand, function (err, stdout) {
      if (err) {
        return callback(err)
      }
      callback(null, removeLineBreaks(stdout.toString()));
    });
  } else {
    try {
      var stdout = execSync(gitCommand);
      return removeLineBreaks(stdout.toString());
    } catch (e) {
      console.error(("Rollup plugin add git msg warn: No git " + command + " yet."));
    }
  }
};

var packageJSON;

var getPackageJSON = function (opts) {
  if(!packageJSON) {
    try {
      packageJSON = readFileSync(path.join(opts.cwd, 'package.json'), 'utf-8');
      packageJSON = JSON.parse(packageJSON);
    } catch (e) {
      console.error(("Rollup plugin add git msg warn: Cannot found " + (opts.cwd) + "/package.json."));
    }
  }

  return packageJSON;
};

var COMMITHASH_COMMAND = 'rev-parse HEAD';
var TAG_COMMAND = 'describe --tags';

var COMMENT_LINE_START = '* ';
var NEW_LINE_CHAR = '\n';

var DEFAULT_OPTIONS = {
  showDate: true,
  showTag: true,
  showCommitID: true,
  cwd: process.cwd(),
};

var Banner = function Banner(options) {
  if ( options === void 0 ) options = {};

  this.options = Object.assign(DEFAULT_OPTIONS, options);
};

Banner.prototype.getHeaderArray = function getHeaderArray (opt) {
  var header = [];
  var gitMsg = '',
    date = new Date();

  var packageJSON = getPackageJSON(opt);
  if (packageJSON.name) {
    gitMsg = gitMsg.concat(("${" + (packageJSON.name) + "} "));
  }

  if (opt.showTag) {
    var tag = runGitCommand(TAG_COMMAND);
    if (tag) {
      var prefix = tag.startsWith('v') ? '' : 'v';
      gitMsg = gitMsg.concat(("" + prefix + tag + " - "));
    }
  }

  if (opt.showCommitID) {
    var commitID = runGitCommand(COMMITHASH_COMMAND);
    if (commitID) {
      gitMsg = gitMsg.concat(("${" + commitID + "}"));
    }
  }

  header.push(gitMsg);
  header.push('');

  if (opt.copyright) {
    header.push(("Copyright " + (date.getFullYear()) + " by " + (opt.copyright)));
    header.push('');
  }

  if (opt.showDate) {
    header.push(("Date: " + (date.toISOString().substring(0, 10))));
    header.push('');
  }
  return header;
};

Banner.prototype.getHeader = function getHeader () {
  var header = this.getHeaderArray(this.options);
  var comment = header.map(function (headerpart) {
    return headerpart.split(DEFAULT_OPTIONS.newLineChar)
      .map(function (line) {
        return ("" + COMMENT_LINE_START + line)
      })
      .join(NEW_LINE_CHAR)
  })
  .join(NEW_LINE_CHAR);

  var commentWithLine = "/*!" + NEW_LINE_CHAR + comment + "*/";

  return commentWithLine;
};

function addGitMsg(options) {
  if ( options === void 0 ) options = {};

  return {
    name: 'addGitMsg',

    banner: function banner() {
      var banner = new Banner(options);
      return banner.getHeader();
    }
  };
}

export default addGitMsg;
